;;;
;;; package.lisp
;;;
;;; Package definition for assembler.
;;;

(cl:defpackage :esb-assem
  (:use :common-lisp :esb-utils)
  (:export
   "ASSEMBLY-UNIT"
   
   "*ASSEM-SCHEDULER-P*"
   "*ASSEM-INSTRUCTIONS*"
   "*ASSEM-MAX-LOCATIONS*"

   "EMIT-BYTE" "EMIT-SKIP" "EMIT-BACK-PATCH"
   "EMIT-CHOOSER" "DEFINE-BITFIELD-EMITTER"
   "DEFINE-INSTRUCTION" "DEFINE-INSTRUCTION-MACRO"
   "DEF-ASSEMBLER-PARAMS" "EMIT-POSTIT"

   "MAKE-SEGMENT" "SEGMENT-NAME" "ASSEMBLE"
   "ALIGN" "INST" "LABEL" "LABEL-P" "GEN-LABEL"
   "EMIT-LABEL" "LABEL-POSITION" "APPEND-SEGMENT" "FINALIZE-SEGMENT"
   "ON-SEGMENT-CONTENTS-VECTORLY" "WRITE-SEGMENT-CONTENTS"
   "READS" "WRITES" "SEGMENT"
   "WITHOUT-SCHEDULING"
   "VARIABLE-LENGTH"
   "SEGMENT-COLLECT-DYNAMIC-STATISTICS"

   "FLUSHABLE" "BRANCH"))

;;; EOF
