;;;
;;; utils/package.lisp
;;;
;;; Package definition for support utilities.
;;;

(cl:defpackage :esb-utils
  (:use :common-lisp)
  (:export

   ;; The "ultimate collection macro" from collect.lisp.
   "COLLECT"

   ;; Automatic structure printer definition macro from defprinter.lisp.
   "DEFPRINTER"

   ;; Boolean attributes (bitwise packed fields) from boolean-attributes.lisp.
   "DEF-BOOLEAN-ATTRIBUTE" "ATTRIBUTES"

   ;; Miscellaneous utilities from misc.lisp.
   "SYMBOLICATE" "ONCE-ONLY" "WITH-UNIQUE-NAMES"

   ;; Sparse set implementation from sset.lisp.
   "DO-SSET-ELEMENTS" "MAKE-SSET" "SSET"
   "SSET-ADJOIN" "SSET-DELETE" "SSET-ELEMENT" "SSET-EMPTY"))

;;; EOF
